@@ -292,6 +292,7 @@ embeddingGroupPlot <- function(plot.df, groups, geom_point_w, min.cluster.size,
 #' @param geom_point_w function to work with geom_point layer from ggplot2 (default=ggplot2::geom_point)
 #' @return ggplot2 object
 embeddingColorsPlot <- function(plot.df, colors, groups=NULL, geom_point_w=ggplot2::geom_point, gradient.range.quantile=1, color.range="symmetric", legend.title=NULL, palette=NULL, plot.na=TRUE) {
+  
   plot.df <- plot.df %>% dplyr::mutate(Color=colors[.data$CellName])
   if(!is.null(groups)) {
     plot.df$Color[!plot.df$CellName %in% names(groups)] <- NA
@@ -367,10 +368,22 @@ styleEmbeddingPlot <- function(gg, plot.theme=NULL, title=NULL, legend.position=
   return(gg)
 }
 
+#' embeddingPlot generic
+#' Plot embedding with provided labels / colors using ggplot2
+#'
+#' @param object two-column matrix with x and y coordinates of the embedding, rownames contain cell names and are used to match coordinates with groups or colors
+#' @param ... Additional argument list, see 'embeddingPlot()' methods for details
+#'
+#' @docType methods
+#' @rdname embeddingPlot-methods
+#' @export
+setGeneric("embeddingPlot", function(object, ...) standardGeneric("embeddingPlot"))
+
+
 #' Plot embedding with provided labels / colors using ggplot2
 #'
 #' @inheritDotParams ggrepel::geom_label_repel
-#' @param embedding two-column matrix with x and y coordinates of the embedding, rownames contain cell names and are used to match coordinates with groups or colors
+#' @param object two-column matrix with x and y coordinates of the embedding, rownames contain cell names and are used to match coordinates with groups or colors
 #' @param groups vector of cluster labels, names contain cell names (default=NULL)
 #' @param colors vector of numbers, which must be shown with point colors, names contain cell names (default=NULL). This argument is ignored if groups are provided.
 #' @param subgroups subset of 'groups', selecting the cells for plot (default=NULL). Ignored if 'groups' is NULL
@@ -399,12 +412,16 @@ styleEmbeddingPlot <- function(gg, plot.theme=NULL, title=NULL, legend.position=
 #' library(sccore)
 #' embeddingPlot(umapEmbedding, show.ticks=TRUE, show.labels=TRUE, title="UMAP embedding")
 #'
+#' @rdname embeddingPlot-methods
+#' @aliases embeddingPlot-methods,ANY
 #' @export
-embeddingPlot <- function(embedding, groups=NULL, colors=NULL, subgroups=NULL, plot.na=is.null(subgroups), min.cluster.size=0, mark.groups=TRUE,
-                          show.legend=FALSE, alpha=0.4, size=0.8, title=NULL, plot.theme=NULL, palette=NULL, color.range="symmetric",
-                          font.size=c(3, 7), show.ticks=FALSE, show.labels=FALSE, legend.position=NULL, legend.title=NULL,
-                          gradient.range.quantile=1, raster=FALSE, raster.dpi=300, shuffle.colors=FALSE, keep.limits=!is.null(subgroups), ...) {
-  plot.df <- tibble::rownames_to_column(as.data.frame(embedding), "CellName")
+setMethod("embeddingPlot", "ANY", function(
+  object, groups=NULL, colors=NULL, subgroups=NULL, plot.na=is.null(subgroups), min.cluster.size=0, mark.groups=TRUE,
+  show.legend=FALSE, alpha=0.4, size=0.8, title=NULL, plot.theme=NULL, palette=NULL, color.range="symmetric",
+  font.size=c(3, 7), show.ticks=FALSE, show.labels=FALSE, legend.position=NULL, legend.title=NULL,
+  gradient.range.quantile=1, raster=FALSE, raster.dpi=300, shuffle.colors=FALSE, keep.limits=!is.null(subgroups), ...
+) {
+  plot.df <- tibble::rownames_to_column(as.data.frame(object), "CellName")
   colnames(plot.df)[2:3] <- c("x", "y")
 
   if (raster && requireNamespace("ggrastr", quietly = TRUE)) {
@@ -442,13 +459,45 @@ embeddingPlot <- function(embedding, groups=NULL, colors=NULL, subgroups=NULL, p
   }
 
   if(keep.limits) {
-    gg <- gg + ggplot2::lims(x=range(embedding[,1]), y=range(embedding[,2]))
+    gg <- gg + ggplot2::lims(x=range(object[,1]), y=range(object[,2]))
   }
 
   gg <- styleEmbeddingPlot(gg, plot.theme=plot.theme, title=title, legend.position=legend.position,
                            show.legend=show.legend, show.ticks=show.ticks, show.labels=show.labels)
   return(gg)
-}
+})
+
+
+#' Plot embedding from Seurat object
+#'
+#' @param object Seurat object
+#' @param reduction Reduction to use for embedding (default=NULL)
+#' @return ggplot2 object
+#' @examples
+#' \dontrun{
+#' # so = example Seurat object
+#' embeddingPlot(so, groups="seurat_clusters", reduction="umap")
+#' }
+#'
+#' @rdname embeddingPlot-methods
+#' @aliases embeddingPlot-methods,Seurat
+#' @export
+setMethod("embeddingPlot", signature("Seurat"), function(object, reduction=NULL, groups=NULL, colors=NULL, ...) {
+  if (is.null(reduction)) {
+    reduction <- Seurat::Reductions(object)[1]
+  }
+
+  if (!is.null(groups) && length(groups) == 1) {
+      groups <- setNames(unlist(object[[groups]]), colnames(object))
+  }
+
+  if (!is.null(colors) && length(colors) == 1) {
+      colors <- setNames(unlist(object[[colors]]), colnames(object))
+  }
+
+  embeddingPlot(Seurat::Embeddings(object, reduction), groups=groups, colors=colors, ...)
+})
+
 
 #' Dot plot adapted from Seurat:::DotPlot, see ?Seurat:::DotPlot for details
 #'
